#https://fedorahosted.org/sssd/wiki/pcech_test_contribute

[[PageOutline]]

= Contribute =

There are many ways you can contribute to the System Security Services Daemon. This page should provide some basic pointers.

SSSD development discussions occur either on the [https://fedorahosted.org/mailman/listinfo/sssd-devel SSSD development list] or on the [irc://irc.freenode.net/sssd #sssd IRC channel] on [http://freenode.net/ freenode]. Keep in mind that SSSD developers are spread across different time zones, which makes mailing list preferred communication channel if you wish to reach the entire team.

The SSSD originated as a sister-project to [http://www.freeipa.org FreeIPA], so we share many pieces. Contributing to FreeIPA may also help SSSD and vice versa.

If you want to file a bug or enhancement request, please log in with your Fedora Account System credentials. If you do not have a Fedora Account, you can register for one at https://admin.fedoraproject.org/accounts/. There is dedicated page about [https://fedorahosted.org/sssd/wiki/Reporting_sssd_bugs bug reporting].

== Contribution Policy ==
All source code committed to the SSSD is assumed to be made available under the GPLv3+ license unless the submitter specifies another license at that time. The upstream SSSD maintainers reserve the right to refuse a submission if the license is deemed incompatible with the goals of the project.

== Source Code Repository ==

For the SSSD project, we use the [http://www.kernel.org/pub/software/scm/git/docs/gittutorial.html Git] source control system.

{{{
git://git.fedorahosted.org/git/sssd.git
}}}

=== Getting the source ===

To check out the latest SSSD sources, install git on your system (Instructions are for Fedora, you may need to use a platform-specific installation mechanism for other OSes).
{{{
sudo yum -y install git
}}}
You can find an excellent [http://www.kernel.org/pub/software/scm/git/docs/gittutorial.html tutorial] for using git.

The first time, you will want to configure git with your name and email address for patch attribution:
{{{
git config --global user.name "GIVENNAME SURNAME"
git config --global user.email "username@domain.com"
}}}

You can enable syntax highlighting at the console (for git commands such as '`git status`' and '`git diff`') by setting:
{{{
git config --global color.ui auto
}}}
And if you would like to have a graphical tool for resolving merge conflicts, you can install and set up meld:
{{{
sudo yum -y install meld
}}}
{{{
git config --global merge.tool meld
}}}
It can be invoked with
{{{
git mergetool
}}}
You can also enable the commit template by:
{{{
git config commit.template .git-commit-template
}}}

Once that's done, you can clone our upstream git repository with
{{{
git clone git://git.fedorahosted.org/git/sssd.git
}}}

This will create a new subdirectory 'sssd' in the current directory which will contain the current master sources. All development should be done first against the master branch, then backported to one of the stable branches if necessary.

== Building SSSD ==
Starting with SSSD 1.10 beta, we now include a set of helper aliases and environment variables in SSSD to simplify building development versions of SSSD on Fedora. To take advantage of them, use the following command:
{{{
. /path/to/sssd-source/contrib/fedora/bashrc_sssd
}}}

To build SSSD from source, follow these steps:

1. Install necessary tools
{{{
# when using yum
sudo yum -y install rpm-build yum-utils

# when using dnf
sudo dnf -y install rpm-build dnf-plugins-core
}}}

2. Create source rpm (run from git root directory)
{{{
contrib/fedora/make_srpm.sh
}}}

3. Install SSSD dependencies
{{{
# when using yum
sudo yum-builddep rpmbuild/SRPMS/sssd-*.src.rpm

# when using dnf
sudo dnf builddep rpmbuild/SRPMS/sssd-*.src.rpm
}}}

If you plan on working with the SSSD source often, you may want to add the following to your {{{~/.bashrc}}} file:
{{{
if [ -f /path/to/sssd-source/contrib/fedora/bashrc_sssd ]; then
    . /path/to/sssd-source/contrib/fedora/bashrc_sssd
fi
}}}

You can now produce a Debug build of SSSD by running:
{{{
cd /path/to/sssd-source
reconfig && chmake
}}}

The {{{reconfig}}} alias will run {{{autoreconf -if}}}, create a parallel build directory named after your CPU architecture (e.g. x86_64) and then run the configure script with the appropriate options for installing on a Fedora system with all experimental features enabled.

The {{{chmake}}} alias will then perform the build (with build messages suppressed except on warning or error) as well as build and run tests.

Finally, assuming you have sudo privilege, you can run the {{{sssinstall}}} alias to install this debug build onto the build machine.

To install SSSD on other distributions, you can run the usual Autotools combo:
{{{
autoreconf -i -f && \
./configure --enable-nsslibdir=/lib64 --enable-pammoddir=/lib64/security && \
make
sudo make install
}}}

To build and install the code. Please note that by default, the Autotools install prefix is `/usr/local`. Also, if you are building and installing on a 32bit machine, you should use `/lib/` instead of `/lib64` for `nsslibdir` and `pammoddir`. Please note that even if you are installing to `/usr/local`, the NSS and PAM libraries must be installed to system library directories as that's where NSS and PAM would be looking for them.

=== Tips and tricks for developers ===
There is a [DevelTips dedicated page] that contains a collection of tips that are useful for developers.


=== COPR Repository ===
You can download development packages from [https://copr.fedoraproject.org/coprs/ COPR] repo:
 * https://copr.fedoraproject.org/coprs/lslebodn/

== Sending patch to upstream ==

=== Coding Style ===
We have adopted the code style and formatting specification used by the FreeIPA project to describe our [http://www.freeipa.org/page/Coding_Style C] and [http://www.freeipa.org/page/Python_Coding_Style Python] coding styles.

=== Submitting ===

Make your changes and then add any new or modified files to a change-set with the command:
{{{
git add <path_to_file1> <path_to_file2> ...
git commit
}}}

Before submitting a patch, always make sure it applies to the latest upstream master branch. You will want to rebase to this branch and fix any merge conflicts (in case someone else changed the same code).
{{{
git remote update
git rebase -i origin/master
}}}

If this rebase has a merge conflict, you will need to resolve the conflict before you continue. If you get stuck or make a mistake, you can use
{{{
git rebase --abort
}}}
This will put you right back where you started.

Once your changes are ready for submission, you can use the following command to generate a patch
{{{
git format-patch -M -C --patience --full-index -1
}}}

This will make a local commit (it doesn't get pushed upstream) and then creates a patch locally. You can create multiple commits and then change {{{-1}}} to {{{-N}}} where N is the number of commits to produce patches for. To submit your patch for code-review and inclusion upstream, mail it as an attachment to {{{sssd-devel@lists.fedorahosted.org}}}.

If a patch isn't accepted on the first review, you will need to modify it and resubmit it. When this happens, you will want to amend your changes to the existing patch with
{{{
git add <files>
git commit --amend
}}}

If you need to make changes to earlier patches in your tree, you can use
{{{
git rebase -i origin/master
}}}
and follow the directions in the interactive rebase to modify specific patches.

=== Patch metadata ===
The description associated with the patch is an important piece of information that allows other developers or users to see what the change was about, what bug did the commit fix or what feature did the commit implement. To structure the information many SSSD developers use the following format:

* one-line short description
* blank line
* ticket or bugzilla URL (if available)
* blank line
* One or more paragraphs that describe the change if it can't be described fully in the one-line description

These best practices are loosely based on the [http://www.kernel.org/doc/Documentation/SubmittingPatches kernel patch submission recommendation].

An example of a patch formatted according to the above guidelines is commit e4d1e192430f6c76eee5b64decc3a799a0926979:
{{{
krb: recreate ccache if it was deleted

https://fedorahosted.org/sssd/ticket/1512

If directory where a ccache file was stored was missing and user
was still logged in, we erroneously considered the ccache file
still active. Thus the ccache file was not recreated and user was
unable to login.
}}}

=== Unit tests ===
It is preferred if a patch comes together with a unit test. SSSD uses the [http://cmocka.org/ cmocka] library for developing unit tests, although some older tests still use the [http://check.sourceforge.net/ check framework]. For an introduction to cmocka, see either the examples in the `src/tests/cmocka` directory or read the [https://lwn.net/Articles/558106/ lwn article on cmocka].

=== Localization and Internationalization ===

Our development policy for the SSSD requires that any code that generates a user-facing message should be wrapped by GNU {{{gettext}}} macros so that they can eventually be translated. We use [http://zanata.org/ zanata] for translating.

= Latest Documentation and Presentations =

There is a dedicated page where we keep our documentation.

 * [wiki:Documentation Documentation]
